import processing.core.*; 
import processing.xml.*; 

import controlP5.*; 
import peasy.*; 
import processing.opengl.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class Balloon extends PApplet {





PeasyCam cam;
ControlP5 controlP5;
ControlWindow controlWindow;

int totalSegments = 10;

PVector[] orbit = new PVector[totalSegments];
Segment[] segments = new Segment[totalSegments];

PVector segmentOffset;

float margins = .1f;
float rootScalar = 100;
float rootRadius = 230;

float rootHighlightSpeed = .5f;
float rootSpinSpeed = .1f;

float elastictySmoothness = .7f;
float elastictyRigidity = .2f;
float elastictySpread = 3;

Boolean isShape;
Boolean isHighlight;
Boolean isMesh;
Boolean isWire;
Boolean isGuide;
Boolean isAxis;


public void setup() {
	size(1200,800,OPENGL); 
  
    cam = new PeasyCam(this, 1000);
    cam.setMinimumDistance(50);
    cam.setMaximumDistance(2000);
    cam.lookAt(0, 0, 0, 110);
    cam.setActive(true);  // false to make this camera stop responding to mouse
    
    controlP5 = new ControlP5(this);                       

    /*
    controlWindow = controlP5.addControlWindow("controlP5window",100,100,300,600);
    controlWindow.hideCoordinates();
    controlWindow.setBackground(color(40));
    */
	
    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------
    int offset;
    int id = 0;
    float absMax = 100;
    int sliderWidth  = 190;
    int sliderHeight = 12;
    int sliderHeightSpread = 20;
    offset = 50;
    controlP5.addSlider("margins", -1,1,margins,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;

    controlP5.addSlider("rootScalar", 0,200,rootScalar,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    controlP5.addSlider("rootRadius", 0,1000,rootRadius,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;
    controlP5.addSlider("rootHighlightSpeed", 0,1,rootHighlightSpeed,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;
    controlP5.addSlider("rootSpinSpeed", 0,5,rootSpinSpeed,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;
    controlP5.addSlider("elastictySmoothness", 0,1,elastictySmoothness,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    controlP5.addSlider("elastictyRigidity", 0,1,elastictyRigidity,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    controlP5.addSlider("elastictySpread", 0,10,elastictySpread,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;

    controlP5.addButton("resetButton", 0,    10,offset+id*sliderHeightSpread, sliderWidth,30); id++;

	controlP5.setAutoDraw(false);

	toggleShape(false);
    toggleWire(false);              
    toggleGuide(false);
    toggleAxis(true);
    toggleHighlight(false);
    toggleMesh(true);                          

   	float inc = 70;
    controlP5.addToggle("toggleShape",isShape,10+inc*0,10,50,20);
	controlP5.addToggle("toggleWire",isWire,10+inc*1,10,50,20);
	controlP5.addToggle("toggleGuide",isGuide,10+inc*2,10,50,20);
	controlP5.addToggle("toggleAxis",isAxis,10+inc*3,10,50,20);
	controlP5.addToggle("toggleHighlight",isHighlight,10+inc*4,10,50,20);
	controlP5.addToggle("toggleMesh",isMesh,10+inc*5,10,50,20);
	
    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------

	for(int i=0;i<totalSegments;i++){
		segments[i] = new Segment();
		if(i==0){
			segments[i].previousSegment = segments[0];
		} else {
			segments[i].previousSegment = segments[i-1];
		}
		segments[i].seed = i;
		segments[i].normalizedSeed = PApplet.parseFloat(i+1)/PApplet.parseFloat(totalSegments);
		
		segments[i].randomizeDirection();
		segments[i].createRelatives();
		segments[i].resetRelatives();
		segments[i].createAbsolutes();
	}

}

public void resetButton(int theValue) {
	reset();
}
public void parseContent(){
	int i;

	segmentOffset = new PVector(0,0,0);
	
	for(i=0;i<totalSegments;i++){
		segments[i].setControlParams();
		segments[i].createRelatives();
		segments[i].resetRelatives();
		calculateSegmentPosition(i);
	}
	
	// first
	i = 0;
	segments[i].setPreviousOrbit(segments[i].startOrbit);
	segments[i].setNextOrbit(segments[i+1].startOrbit);
	segments[i].parseLerps();
	// middle
	for(i=1;i<(totalSegments-1);i++){
		segments[i].setPreviousOrbit(segments[i-1].endOrbit);
		segments[i].setNextOrbit(segments[i+1].startOrbit);
		segments[i].parseLerps();
	}
	// last
	i = totalSegments-1;
	segments[i].setPreviousOrbit(segments[i-1].endOrbit);
	segments[i].setNextOrbit(segments[i].endOrbit);
	segments[i].parseLerps();
	
}
public void drawContent() {
	int i;
	if(isShape){
		for(i=0;i<totalSegments;i++){
			segments[i].drawShapes();
		}
	}
	if(isHighlight){
		for(i=0;i<totalSegments;i++){
			segments[i].drawHighlights();
		}
	}
	if(isMesh){
		for(i=0;i<totalSegments;i++){
			segments[i].drawMesh();
		}
	}
	if(isWire){
		for(i=0;i<totalSegments;i++){
			segments[i].drawWireframes();
		}
	}
	if(isGuide){
		for(i=0;i<totalSegments;i++){
			segments[i].drawGuides();
		}
	}
	
	if(isAxis){
		drawAxes(400);
	}
}

public void controlEvent(ControlEvent theEvent) {
    cam.setActive(false);  // false to make this camera stop responding to mouse
}

public void mouseReleased() {
    cam.setActive(true);  // false to make this camera stop responding to mouse
}

public void toggleShape(boolean theFlag) {
	isShape = theFlag;
}
public void toggleHighlight(boolean theFlag) {
	isHighlight = theFlag;
}
public void toggleMesh(boolean theFlag) {
	isMesh = theFlag;
}
public void toggleWire(boolean theFlag) {
	isWire = theFlag;
}
public void toggleGuide(boolean theFlag) {
	isGuide = theFlag;
}
public void toggleAxis(boolean theFlag) {
	isAxis = theFlag;
}

public void draw() {

	background(5);

	cam.beginHUD();
		if(isAxis){
			noFill();
			strokeWeight(1.0f);
			stroke(0xffFFFFFF,25);
			ellipse(0, 0, 200, 200);
			ellipse(0, 0, 180, 180);
		}
		controlP5.draw();
	cam.endHUD();
  
	lights();

	parseContent();
	drawContent();
    
    
}

public void calculateSegmentPosition(int i){
	segments[i].createAbsolutes();
	segments[i].updateAbsolutes(segmentOffset);
	segments[i].parse();     
	segmentOffset.add(segments[i].endRelativeVec.get());
}



public void keyPressed() {
	if (key == 'r' || key == 'R') {
		reset();
	}
}

public void reset(){
	for(int i=0;i<totalSegments;i++){
		segments[i].randomizeDirection();
		segments[i].createRelatives();
		segments[i].setControlParams();
		segments[i].resetRelatives();
		segments[i].createAbsolutes();
	}
}
public void drawAxes(float radius) {
  noFill();
  strokeWeight(1);
  stroke(255, 0, 0,255);
  line(0, 0, 0, radius, 0, 0);
  stroke(0, 255, 0,255);
  line(0, 0, 0, 0, radius, 0);
  stroke(0, 0, 255,255);
  line(0, 0, 0, 0, 0, radius);

  stroke(0xff00FF00,51);
  pushMatrix();
  translate(0, radius,0);
  box(10);
  popMatrix();             

  stroke(0xffFF0000,51);
  pushMatrix();
  translate(radius, 0,0);
  box(10);
  popMatrix(); 

  stroke(0xff0000FF,51);
  pushMatrix();
  translate(0, 0,radius);
  box(10);
  popMatrix(); 

  stroke(0xffFFFFFF,51);
  ellipse(0, 0, 200, 200);

  stroke(0xffFFFFFF,51);
  box(radius*2);
}

public Quaternion quat_mult(Quaternion q1, Quaternion q2) {
	float a = (q1.a*q2.a - q1.b*q2.b - q1.c*q2.c - q1.d*q2.d);
	float b = (q1.a*q2.b + q1.b*q2.a + q1.c*q2.d - q1.d*q2.c);
	float c = (q1.a*q2.c - q1.b*q2.d + q1.c*q2.a + q1.d*q2.b);
	float d = (q1.a*q2.d + q1.b*q2.c - q1.c*q2.b + q1.d*q2.a);
	Quaternion result = new Quaternion(a,b,c,d);
	return result;
}

// Quaternion multiplication without the .a component
public PVector quat_pointmult( Quaternion q1, Quaternion q2) {
	float x = (q1.a*q2.b + q1.b*q2.a + q1.c*q2.d - q1.d*q2.c);
	float y = (q1.a*q2.c - q1.b*q2.d + q1.c*q2.a + q1.d*q2.b);
	float z = (q1.a*q2.d + q1.b*q2.c - q1.c*q2.b + q1.d*q2.a);
	PVector result = new PVector(x,y,z);
	return result;
}

class Quaternion {
	public float a = 0;
	public float b = 0;
	public float c = 0;
	public float d = 1;
	
	public Quaternion(float aa,float bb,float cc,float dd) {
		this.a = aa;
		this.b = bb;
		this.c = cc;
		this.d = dd;
	}
}
// linear interpolation between vectors
public PVector lerp3D(PVector a, PVector b, float t) {
	float x = lerp(a.x, b.x, t);
	float y = lerp(a.y, b.y, t);
	float z = lerp(a.z, b.z, t); 
	return new PVector(x,y,z); 
}

// linear interpolation between vectors
public PVector slerp(PVector a, PVector b, float t) {

	Quaternion q0 = new Quaternion(0,a.x,a.y,a.z);
	Quaternion q1 = new Quaternion(0,b.x,b.y,b.z);
	
	float cosTheta = q0.a*q1.a + q0.b*q1.b + q0.c*q1.c + q0.d*q1.d; 
	
	if(cosTheta < 0){
		q1.a = -q1.a;
		q1.b = -q1.b;
		q1.c = -q1.c;
		q1.d = -q1.d;
		cosTheta = -cosTheta;
	}
	
	float k0,k1;
	float sinTheta;
	float theta;
	
	if(cosTheta>.99999f){
		k0 = 1-t;
		k1 = t;
	} else {
		sinTheta = sqrt(1-cosTheta*cosTheta);
		theta = atan2(sinTheta,cosTheta);
		
		k0 = sin((1-t) * theta) /sinTheta;
		k1 = sin(t*theta)/sinTheta;
	}

	float qa = q0.a*k0 + q1.a*k1;
	float qb = q0.b*k0 + q1.b*k1;
	float qc = q0.c*k0 + q1.c*k1;
	float qd = q0.d*k0 + q1.d*k1;

	return new PVector(qb,qc,qd);	
}

public PVector rotateAroundArbitraryAxis(PVector offsetParam,PVector controlParam,PVector axisParam,float angle) {
	PVector control = controlParam.get();
	PVector axis = axisParam.get();
	PVector offset = offsetParam.get();

	control.sub(offset);
	axis.sub(offset);
	axis.normalize(axis);

	float rads = PI/180 * angle;                       
	float sinTheta = sin(rads*.5f);
	float cosTheta = cos(rads*.5f);

	Quaternion rotQuat   = new Quaternion(  cosTheta,  
	sinTheta * axis.x,  
	sinTheta * axis.y,  
	sinTheta * axis.z );
	Quaternion pointQuat = new Quaternion(  0,
	control.x,
	control.y,
	control.z );
	Quaternion conjQuat  = new Quaternion(  rotQuat.a,
	-rotQuat.b,
	-rotQuat.c,
	-rotQuat.d );

	Quaternion temp = quat_mult(rotQuat,pointQuat);
	PVector result = quat_pointmult(temp,conjQuat);
	result.add(offset);
	return result;
}

public PVector rotateAroundZAxis(PVector offset, PVector controlParam, float angle) {

	PVector control = controlParam.get();
	control.sub(offset);

	float rads = PI/180 * angle;
	float cosTheta = cos(rads);
	float sinTheta = sin(rads);

	float x = cosTheta*control.x + sinTheta*control.y;
	float y = -sinTheta*control.x + cosTheta*control.y;
	float z = control.z;

	PVector result = new PVector(x,y,z);
	result.add(offset);

	return result;
}

public PVector rotateAroundYAxis(PVector offset, PVector controlParam, float angle) {
	PVector control = controlParam.get();
	control.sub(offset);

	float rads = PI/180 * angle;
	float cosTheta = cos(rads);
	float sinTheta = sin(rads);

	float x = cosTheta*control.x - sinTheta*control.z;
	float y = control.y;
	float z = sinTheta*control.x + cosTheta*control.z;

	PVector result = new PVector(x,y,z);

	result.add(offset);

	return result;
}

public PVector rotateAroundXAxis(PVector offset, PVector controlParam, float angle) {
	PVector control = controlParam.get();
	control.sub(offset);

	float rads = PI/180 * angle;
	float cosTheta = cos(rads);
	float sinTheta = sin(rads);

	float x = control.x;
	float y = cosTheta*control.y + sinTheta*control.z;
	float z = -sinTheta*control.y + cosTheta*control.z;

	PVector result = new PVector(x,y,z);

	result.add(offset);

	return result;
}


class Segment {

	public int seed;
	public float normalizedSeed;

	public float angle;

	public int totalLines = 25;

	public PVector[] baseOrbit = new PVector[totalLines];
	public PVector[] startOrbit = new PVector[totalLines];
	public PVector[] startMarginOrbit = new PVector[totalLines];
	public PVector[] endOrbit = new PVector[totalLines];
	public PVector[] endMarginOrbit = new PVector[totalLines];
	public PVector[] midOrbit = new PVector[totalLines];
	public PVector[] prevStartOrbit = new PVector[totalLines];
	public PVector[] nextStartOrbit = new PVector[totalLines];
	public PVector[] startLerpOrbit = new PVector[totalLines];
	public PVector[] endLerpOrbit = new PVector[totalLines];
	
	public PVector zeroVect;

	public PVector startRelativeVec;
	public PVector endRelativeVec;
	public PVector startCrossRelativeVec;
	public PVector endCrossRelativeVec;
	public PVector pseudoRelativeVec;

	public PVector startAbsoluteVec;
	public PVector endAbsoluteVec;
	public PVector startCrossAbsoluteVec;
	public PVector endCrossAbsoluteVec;
	public PVector pseudoAbsoluteVec;

	public Segment previousSegment;

	public PVector randomDirectionVector;
	public PVector currentDirectionVector;

	PVector velocityVec,xVec,targetVec;
	int from = color(255, 0, 102);
	int to = color(0, 102, 153);

	public float scalar = 1;

	public Segment() {
		zeroVect	= new PVector( 0,0,0);
		endRelativeVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
		currentDirectionVector	= new PVector( 0,0,0);

		velocityVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
		xVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
		targetVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
	}

	public void randomizeDirection(){
		// println("randomizeDirection + " + seed);
		randomDirectionVector = new PVector(random(-1,1),random(-1,1),random(-1,1));
	}
	
	public void createRelatives(){
		// print("createRelatives");
		startRelativeVec = new PVector( 0, 0, 0 );
		
		/*
		PVector dif = PVector.sub(randomDirectionVector,currentDirectionVector);
		dif.mult(.025*(totalSegments-seed)*.1);
		currentDirectionVector.add(dif);

		endRelativeVec = currentDirectionVector.get();
		endRelativeVec.mult(rootRadius);
		// endRelativeVec.mult(rootRadius*(seed));
		*/
		// velocityVec,xVec,targetVec;
		
		PVector divVec = PVector.sub(randomDirectionVector,xVec);
		divVec.mult(elastictySmoothness*normalizedSeed*elastictySpread);
		
		velocityVec = PVector.add(velocityVec,divVec);
		velocityVec = PVector.mult(velocityVec,elastictyRigidity);
		
		xVec = PVector.add(velocityVec,xVec);
		
		endRelativeVec = xVec.get();
		endRelativeVec.mult(rootRadius);
		
		
	}

	public void setControlParams(){
		// scalar = mouseY-height/2;// random(50) + 50;
		float noiseVal = noise(frameCount*.0125f,seed*10);
		// scalar = noiseVal*50 + 50;
		scalar = rootScalar + noiseVal*100;
		// scalar = rootScalar*noiseVal;
	}

	public void resetRelatives(){
		pseudoRelativeVec = new PVector( 0, 0, scalar );
		startCrossAbsoluteVec = new PVector( 0, 0, 0 );
		endCrossAbsoluteVec = new PVector( 0, 0, 0 );

		startCrossRelativeVec = endRelativeVec.cross(pseudoRelativeVec);
		startCrossRelativeVec.normalize();
		startCrossRelativeVec.mult(scalar);

		endCrossRelativeVec = startCrossRelativeVec.get();
		endCrossRelativeVec.add(endRelativeVec);

	}
	
	public void createAbsolutes(){
		startAbsoluteVec = startRelativeVec.get();
		endAbsoluteVec = endRelativeVec.get();
		startCrossAbsoluteVec = startCrossRelativeVec.get();
		endCrossAbsoluteVec = endCrossRelativeVec.get();
		pseudoAbsoluteVec = pseudoRelativeVec.get();
	}
	
	public void updateAbsolutes(PVector offset){
		startAbsoluteVec.add(offset);
		endAbsoluteVec.add(offset);
		startCrossAbsoluteVec.add(offset);
		endCrossAbsoluteVec.add(offset);
		pseudoAbsoluteVec.add(offset);
		
	}
	
	public void setPreviousOrbit(PVector[] orbiter){
		prevStartOrbit = orbiter;
	}
	public void setNextOrbit(PVector[] orbiter){
		nextStartOrbit = orbiter;
	}
	
	public void parse() {

		PVector startMarginVec = endRelativeVec.get();
		startMarginVec.mult(margins);
		PVector midVec = endRelativeVec.get();
		midVec.mult(.5f);
		
		PVector endMarginVec = endRelativeVec.get();
		endMarginVec.mult(1-margins);
		 
		for(int i=0;i<totalLines;i++) {
			// angle =  360*float(i)/float(totalLines);
			// angle = 360*float(i)/float(totalLines) +360*(1-float(seed)/float(totalLines));
			// angle = 360*float(i)/float(totalLines) +180*(1-float(seed)/float(totalLines))+ frameCount*.125;
			angle = 360*PApplet.parseFloat(i)/PApplet.parseFloat(totalLines) +360*(1-PApplet.parseFloat(seed)/PApplet.parseFloat(totalLines))+ frameCount*rootSpinSpeed*seed;
			
			startOrbit[i] = rotateAroundArbitraryAxis( startAbsoluteVec, startCrossAbsoluteVec, endAbsoluteVec, angle );
			
			startMarginOrbit[i] = startOrbit[i].get();
			startMarginOrbit[i].add(startMarginVec);

			midOrbit[i] = startOrbit[i].get();
			midOrbit[i].add(midVec);
			
			endMarginOrbit[i] = startOrbit[i].get();
			endMarginOrbit[i].add(endMarginVec);
	
			endOrbit[i] = startOrbit[i].get();
			endOrbit[i].add(endRelativeVec);
			
		}         
	}
	
	public void parseLerps() {
		for(int i=0;i<totalLines;i++) {
			startLerpOrbit[i] = lerp3D(prevStartOrbit[i],startOrbit[i],.5f);
			endLerpOrbit[i] = lerp3D(nextStartOrbit[i],endOrbit[i],.5f);	
			
			// startLerpOrbit[i] = slerp(prevStartOrbit[i],startOrbit[i],.5);
			// endLerpOrbit[i] = slerp(nextStartOrbit[i],endOrbit[i],.5);	
			
		}
	}
	public void drawWireframes() {
		drawCircleOutlineArray(startOrbit,0xffFFFFFF,102);
		drawCircleOutlineArray(endOrbit,0xffFFFFFF,102);

		drawCircleOutlineArray(midOrbit,0xff00FF00,51);
		drawCircleOutlineArray(startMarginOrbit,0xffFF00FF,51);
		drawCircleOutlineArray(endMarginOrbit,0xffFF00FF,51);
		
		drawCirclePointArray(startLerpOrbit,0xffFF0000,102);
		drawCirclePointArray(endLerpOrbit,0xffFF0000,102);
		
		
		drawLine(startAbsoluteVec,pseudoAbsoluteVec,1,0xff00FF00,102);

		drawPoint(startCrossAbsoluteVec,3,0xffFF00FF,255);
		drawLine(startAbsoluteVec,startCrossAbsoluteVec,1,0xffFF00FF,102);

		drawPoint(endCrossAbsoluteVec,3,0xffFFFFFF,255);
		drawLine(endAbsoluteVec,endCrossAbsoluteVec,1,0xffFFFFFF,102);
		
	}
	public void drawShapes() {
		
		int lines = endOrbit.length;
		int i;
		float percentage;
		
		for(i=0;i<lines;i++) {
		
			noFill();
			percentage = 1;//(float(i)/float(lines));
			stroke(0xffFFFFFF,percentage*25);
			strokeWeight(1);
			// curve(x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4);
						
			bezier(
				startAbsoluteVec.x, startAbsoluteVec.y, startAbsoluteVec.z,
				startLerpOrbit[i].x,startLerpOrbit[i].y,startLerpOrbit[i].z,
				startMarginOrbit[i].x,startMarginOrbit[i].y,startMarginOrbit[i].z,
				midOrbit[i].x,midOrbit[i].y,midOrbit[i].z
			);
			
			bezier(
				endAbsoluteVec.x, endAbsoluteVec.y, endAbsoluteVec.z,
				endLerpOrbit[i].x,endLerpOrbit[i].y,endLerpOrbit[i].z,
				endMarginOrbit[i].x,endMarginOrbit[i].y,endMarginOrbit[i].z,
				midOrbit[i].x,midOrbit[i].y,midOrbit[i].z
			);
		
		}                                                        
	}
	public void drawHighlights() {
		
		int lines = endOrbit.length;
		int i;
		float percentage;
		float bx,by,bz;
		float ax,ay,az;
		PVector pointA,pointB;
		PVector midA,midB;
		int interColor;
		for(i=0;i<lines;i++) {
			
			percentage = (PApplet.parseFloat(frameCount)*rootHighlightSpeed/PApplet.parseFloat(lines))%1;
			bx = bezierPoint(startAbsoluteVec.x, startLerpOrbit[i].x, startMarginOrbit[i].x, midOrbit[i].x, percentage	);
			by = bezierPoint(startAbsoluteVec.y, startLerpOrbit[i].y, startMarginOrbit[i].y, midOrbit[i].y, percentage	);
			bz = bezierPoint(startAbsoluteVec.z, startLerpOrbit[i].z, startMarginOrbit[i].z, midOrbit[i].z, percentage	);
			pointB = new PVector(bx,by,bz);

			interColor = lerpColor(0xff99FFFF,0xffFF66FF,  percentage*.5f);
			drawPoint(pointB,2,interColor,255);
			midB = lerp3D(startAbsoluteVec,endAbsoluteVec,percentage*.5f);
			drawLine(midB,pointB,1,interColor,102);
			
			
			ax = bezierPoint(endAbsoluteVec.x, endLerpOrbit[i].x, endMarginOrbit[i].x, midOrbit[i].x, 1-percentage	);
			ay = bezierPoint(endAbsoluteVec.y, endLerpOrbit[i].y, endMarginOrbit[i].y, midOrbit[i].y, 1-percentage	);
			az = bezierPoint(endAbsoluteVec.z, endLerpOrbit[i].z, endMarginOrbit[i].z, midOrbit[i].z, 1-percentage	);
			pointA = new PVector(ax,ay,az);
			interColor = lerpColor(0xff99FFFF,0xffFF66FF,  percentage*.5f+.5f);
			drawPoint(pointA,2,interColor,255);
			midA = lerp3D(startAbsoluteVec,endAbsoluteVec,percentage*.5f+.5f);
			drawLine(midA,pointA,1,interColor,102);
			
		}       
	}
	public void drawMesh() {

		PVector[] sideA;
		PVector[] sideB;
		PVector[] sideC;
		PVector[] sideD;

		int lines = endOrbit.length;
		int dotsPerLines = 10;
		int i,j,a,b;
		float percentage;
		float bx,by,bz;
		
		for(i=0;i<lines;i++) {
			
			sideA = new PVector[dotsPerLines];
			sideB = new PVector[dotsPerLines];
			sideC = new PVector[dotsPerLines];
			sideD = new PVector[dotsPerLines];

			a = i%lines;
			b = (i+1)%lines;
	
			// generates points
			for(j=0;j<(dotsPerLines);j++) {

				percentage = (PApplet.parseFloat(j+1)/PApplet.parseFloat(dotsPerLines));

				bx = bezierPoint(startAbsoluteVec.x, startLerpOrbit[a].x, startMarginOrbit[a].x, midOrbit[a].x, percentage	);
				by = bezierPoint(startAbsoluteVec.y, startLerpOrbit[a].y, startMarginOrbit[a].y, midOrbit[a].y, percentage	);
				bz = bezierPoint(startAbsoluteVec.z, startLerpOrbit[a].z, startMarginOrbit[a].z, midOrbit[a].z, percentage	);
				sideA[j] = new PVector(bx,by,bz);

				bx = bezierPoint(startAbsoluteVec.x, startLerpOrbit[b].x, startMarginOrbit[b].x, midOrbit[b].x, percentage	);
				by = bezierPoint(startAbsoluteVec.y, startLerpOrbit[b].y, startMarginOrbit[b].y, midOrbit[b].y, percentage	);
				bz = bezierPoint(startAbsoluteVec.z, startLerpOrbit[b].z, startMarginOrbit[b].z, midOrbit[b].z, percentage	);
				sideB[j] = new PVector(bx,by,bz);
				
				bx = bezierPoint(endAbsoluteVec.x, endLerpOrbit[a].x, endMarginOrbit[a].x, midOrbit[a].x, percentage	);
				by = bezierPoint(endAbsoluteVec.y, endLerpOrbit[a].y, endMarginOrbit[a].y, midOrbit[a].y, percentage	);
				bz = bezierPoint(endAbsoluteVec.z, endLerpOrbit[a].z, endMarginOrbit[a].z, midOrbit[a].z, percentage	);
				sideC[j] = new PVector(bx,by,bz);

				bx = bezierPoint(endAbsoluteVec.x, endLerpOrbit[b].x, endMarginOrbit[b].x, midOrbit[b].x, percentage	);
				by = bezierPoint(endAbsoluteVec.y, endLerpOrbit[b].y, endMarginOrbit[b].y, midOrbit[b].y, percentage	);
				bz = bezierPoint(endAbsoluteVec.z, endLerpOrbit[b].z, endMarginOrbit[b].z, midOrbit[b].z, percentage	);
				sideD[j] = new PVector(bx,by,bz);

			}
			
			noStroke();
			fill(0xff99FFFF,255);

			beginShape(QUAD_STRIP); 
			vertex(startAbsoluteVec.x,startAbsoluteVec.y,startAbsoluteVec.z);
			vertex(startAbsoluteVec.x,startAbsoluteVec.y,startAbsoluteVec.z);
			for(j=0;j<dotsPerLines;j++) {
				percentage = (PApplet.parseFloat(j+1)/PApplet.parseFloat(dotsPerLines));
				int interColor = lerpColor(0xff99FFFF,0xffFF66FF,  percentage);
				fill(interColor,255);
				vertex(sideA[j].x,sideA[j].y,sideA[j].z);
				vertex(sideB[j].x,sideB[j].y,sideB[j].z);
			}
			endShape(CLOSE);

			fill(0xff99FFFF,255);
			beginShape(QUAD_STRIP);  
			vertex(endAbsoluteVec.x,endAbsoluteVec.y,endAbsoluteVec.z);
			vertex(endAbsoluteVec.x,endAbsoluteVec.y,endAbsoluteVec.z);
			for(j=0;j<dotsPerLines;j++) {
				percentage = (PApplet.parseFloat(j+1)/PApplet.parseFloat(dotsPerLines));
				int interColor = lerpColor(0xff99FFFF,0xffFF66FF,  percentage);
				fill(interColor,255);

				vertex(sideC[j].x,sideC[j].y,sideC[j].z);
				vertex(sideD[j].x,sideD[j].y,sideD[j].z);
			}
			endShape(CLOSE);
		}       
	}
	
	public void drawGuides() {
		
		drawPoint(startAbsoluteVec,3,0xffFFFFFF,255);
		drawPoint(endAbsoluteVec,3,0xffFFFFFF,255);
		drawLine(startAbsoluteVec,endAbsoluteVec,1,0xffFFFFFF,153);
	}
	
	public void drawCirclePointArray(PVector[] orbiter, int colorParam,float opacityParam) {
		PVector curr = new PVector(0,0,0);
		int lines = orbiter.length;
		int i;
		float percentage;

		// drawLine(prev,curr,1,#FFFFFF,51);

		for(i=0;i<lines;i++) {
			curr = orbiter[i].get();
			percentage = 1;//(float(i)/float(lines));
			drawPoint(curr,1,colorParam,percentage*opacityParam);
		}
	}
	public void drawCircleOutlineArray(PVector[] orbiter, int colorParam,float opacityParam) {
		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = orbiter.length;
		int i;
		float percentage;

		prev = orbiter[0].get();
		curr = orbiter[lines-1].get();

		for(i=1;i<lines;i++) {
			prev = curr.get();
			curr = orbiter[i].get();
			percentage = 1;//(float(i)/float(lines));
			drawLine(prev,curr,1,colorParam,percentage*opacityParam);
		}
		
	}
	
	public void drawWidthArray(PVector[] orbiter, int colorParam) {

		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = orbiter.length;
		float percentage;
		float colorShifter = sin( ((normalizedSeed + frameCount*.001f) * TWO_PI))*.5f+.5f;
		strokeWeight(1);
		int interColor = lerpColor(from, to,  colorShifter);

		for(int i=1;i<lines;i++) {
			curr = orbiter[i].get();
			prev = orbiter[i-1].get();

			percentage = (PApplet.parseFloat(i)/PApplet.parseFloat(lines)) * 255;

			noStroke();

			fill(interColor, percentage);
			beginShape(TRIANGLE_FAN);
			vertex(prev.x,prev.y,prev.z);
			vertex(curr.x,curr.y,curr.z);
			vertex(startAbsoluteVec.x,startAbsoluteVec.y,startAbsoluteVec.z);
			vertex(prev.x,prev.y,prev.z);
			endShape();

		}
	}

	public void drawPoint(PVector pointParam, int strokeParam, int colorParam, float opacityParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,opacityParam);
		point( pointParam.x,pointParam.y,pointParam.z);
	}

	public void drawLine(PVector pointAParam,PVector pointBParam, float strokeParam, int colorParam, float opacityParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,opacityParam);
		line( pointAParam.x,pointAParam.y,pointAParam.z,pointBParam.x,pointBParam.y,pointBParam.z);
	}
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--present", "--bgcolor=#666666", "--stop-color=#cccccc", "Balloon" });
  }
}
