import controlP5.*;
import peasy.*;
import processing.opengl.*;

PeasyCam cam;
ControlP5 controlP5;
ControlWindow controlWindow;

int totalSegments = 10;

PVector[] orbit = new PVector[totalSegments];
Segment[] segments = new Segment[totalSegments];

PVector segmentOffset;

float margins = .1;
float rootScalar = 100;
float rootRadius = 230;

float rootHighlightSpeed = .5;
float rootSpinSpeed = .1;

float elastictySmoothness = .7;
float elastictyRigidity = .2;
float elastictySpread = 3;

Boolean isShape;
Boolean isHighlight;
Boolean isMesh;
Boolean isWire;
Boolean isGuide;
Boolean isAxis;


void setup() {
	size(1200,800,OPENGL); 
  
    cam = new PeasyCam(this, 1000);
    cam.setMinimumDistance(50);
    cam.setMaximumDistance(2000);
    cam.lookAt(0, 0, 0, 110);
    cam.setActive(true);  // false to make this camera stop responding to mouse
    
    controlP5 = new ControlP5(this);                       

    /*
    controlWindow = controlP5.addControlWindow("controlP5window",100,100,300,600);
    controlWindow.hideCoordinates();
    controlWindow.setBackground(color(40));
    */
	
    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------
    int offset;
    int id = 0;
    float absMax = 100;
    int sliderWidth  = 190;
    int sliderHeight = 12;
    int sliderHeightSpread = 20;
    offset = 50;
    controlP5.addSlider("margins", -1,1,margins,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;

    controlP5.addSlider("rootScalar", 0,200,rootScalar,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    controlP5.addSlider("rootRadius", 0,1000,rootRadius,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;
    controlP5.addSlider("rootHighlightSpeed", 0,1,rootHighlightSpeed,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;
    controlP5.addSlider("rootSpinSpeed", 0,5,rootSpinSpeed,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;
    controlP5.addSlider("elastictySmoothness", 0,1,elastictySmoothness,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    controlP5.addSlider("elastictyRigidity", 0,1,elastictyRigidity,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    controlP5.addSlider("elastictySpread", 0,10,elastictySpread,    10,offset+id*sliderHeightSpread, sliderWidth,sliderHeight); id++;
    offset+=20;

    controlP5.addButton("resetButton", 0,    10,offset+id*sliderHeightSpread, sliderWidth,30); id++;

	controlP5.setAutoDraw(false);

	toggleShape(false);
    toggleWire(false);              
    toggleGuide(false);
    toggleAxis(true);
    toggleHighlight(false);
    toggleMesh(true);                          

   	float inc = 70;
    controlP5.addToggle("toggleShape",isShape,10+inc*0,10,50,20);
	controlP5.addToggle("toggleWire",isWire,10+inc*1,10,50,20);
	controlP5.addToggle("toggleGuide",isGuide,10+inc*2,10,50,20);
	controlP5.addToggle("toggleAxis",isAxis,10+inc*3,10,50,20);
	controlP5.addToggle("toggleHighlight",isHighlight,10+inc*4,10,50,20);
	controlP5.addToggle("toggleMesh",isMesh,10+inc*5,10,50,20);
	
    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------

	for(int i=0;i<totalSegments;i++){
		segments[i] = new Segment();
		if(i==0){
			segments[i].previousSegment = segments[0];
		} else {
			segments[i].previousSegment = segments[i-1];
		}
		segments[i].seed = i;
		segments[i].normalizedSeed = float(i+1)/float(totalSegments);
		
		segments[i].randomizeDirection();
		segments[i].createRelatives();
		segments[i].resetRelatives();
		segments[i].createAbsolutes();
	}

}

void resetButton(int theValue) {
	reset();
}
void parseContent(){
	int i;

	segmentOffset = new PVector(0,0,0);
	
	for(i=0;i<totalSegments;i++){
		segments[i].setControlParams();
		segments[i].createRelatives();
		segments[i].resetRelatives();
		calculateSegmentPosition(i);
	}
	
	// first
	i = 0;
	segments[i].setPreviousOrbit(segments[i].startOrbit);
	segments[i].setNextOrbit(segments[i+1].startOrbit);
	segments[i].parseLerps();
	// middle
	for(i=1;i<(totalSegments-1);i++){
		segments[i].setPreviousOrbit(segments[i-1].endOrbit);
		segments[i].setNextOrbit(segments[i+1].startOrbit);
		segments[i].parseLerps();
	}
	// last
	i = totalSegments-1;
	segments[i].setPreviousOrbit(segments[i-1].endOrbit);
	segments[i].setNextOrbit(segments[i].endOrbit);
	segments[i].parseLerps();
	
}
void drawContent() {
	int i;
	if(isShape){
		for(i=0;i<totalSegments;i++){
			segments[i].drawShapes();
		}
	}
	if(isHighlight){
		for(i=0;i<totalSegments;i++){
			segments[i].drawHighlights();
		}
	}
	if(isMesh){
		for(i=0;i<totalSegments;i++){
			segments[i].drawMesh();
		}
	}
	if(isWire){
		for(i=0;i<totalSegments;i++){
			segments[i].drawWireframes();
		}
	}
	if(isGuide){
		for(i=0;i<totalSegments;i++){
			segments[i].drawGuides();
		}
	}
	
	if(isAxis){
		drawAxes(400);
	}
}

void controlEvent(ControlEvent theEvent) {
    cam.setActive(false);  // false to make this camera stop responding to mouse
}

void mouseReleased() {
    cam.setActive(true);  // false to make this camera stop responding to mouse
}

void toggleShape(boolean theFlag) {
	isShape = theFlag;
}
void toggleHighlight(boolean theFlag) {
	isHighlight = theFlag;
}
void toggleMesh(boolean theFlag) {
	isMesh = theFlag;
}
void toggleWire(boolean theFlag) {
	isWire = theFlag;
}
void toggleGuide(boolean theFlag) {
	isGuide = theFlag;
}
void toggleAxis(boolean theFlag) {
	isAxis = theFlag;
}

void draw() {

	background(5);

	cam.beginHUD();
		if(isAxis){
			noFill();
			strokeWeight(1.0);
			stroke(#FFFFFF,25);
			ellipse(0, 0, 200, 200);
			ellipse(0, 0, 180, 180);
		}
		controlP5.draw();
	cam.endHUD();
  
	lights();

	parseContent();
	drawContent();
    
    
}

void calculateSegmentPosition(int i){
	segments[i].createAbsolutes();
	segments[i].updateAbsolutes(segmentOffset);
	segments[i].parse();     
	segmentOffset.add(segments[i].endRelativeVec.get());
}



void keyPressed() {
	if (key == 'r' || key == 'R') {
		reset();
	}
}

void reset(){
	for(int i=0;i<totalSegments;i++){
		segments[i].randomizeDirection();
		segments[i].createRelatives();
		segments[i].setControlParams();
		segments[i].resetRelatives();
		segments[i].createAbsolutes();
	}
}
void drawAxes(float radius) {
  noFill();
  strokeWeight(1);
  stroke(255, 0, 0,255);
  line(0, 0, 0, radius, 0, 0);
  stroke(0, 255, 0,255);
  line(0, 0, 0, 0, radius, 0);
  stroke(0, 0, 255,255);
  line(0, 0, 0, 0, 0, radius);

  stroke(#00FF00,51);
  pushMatrix();
  translate(0, radius,0);
  box(10);
  popMatrix();             

  stroke(#FF0000,51);
  pushMatrix();
  translate(radius, 0,0);
  box(10);
  popMatrix(); 

  stroke(#0000FF,51);
  pushMatrix();
  translate(0, 0,radius);
  box(10);
  popMatrix(); 

  stroke(#FFFFFF,51);
  ellipse(0, 0, 200, 200);

  stroke(#FFFFFF,51);
  box(radius*2);
}

