// linear interpolation between vectors
public PVector lerp3D(PVector a, PVector b, float t) {
	float x = lerp(a.x, b.x, t);
	float y = lerp(a.y, b.y, t);
	float z = lerp(a.z, b.z, t); 
	return new PVector(x,y,z); 
}

// linear interpolation between vectors
public PVector slerp(PVector a, PVector b, float t) {

	Quaternion q0 = new Quaternion(0,a.x,a.y,a.z);
	Quaternion q1 = new Quaternion(0,b.x,b.y,b.z);
	
	float cosTheta = q0.a*q1.a + q0.b*q1.b + q0.c*q1.c + q0.d*q1.d; 
	
	if(cosTheta < 0){
		q1.a = -q1.a;
		q1.b = -q1.b;
		q1.c = -q1.c;
		q1.d = -q1.d;
		cosTheta = -cosTheta;
	}
	
	float k0,k1;
	float sinTheta;
	float theta;
	
	if(cosTheta>.99999){
		k0 = 1-t;
		k1 = t;
	} else {
		sinTheta = sqrt(1-cosTheta*cosTheta);
		theta = atan2(sinTheta,cosTheta);
		
		k0 = sin((1-t) * theta) /sinTheta;
		k1 = sin(t*theta)/sinTheta;
	}

	float qa = q0.a*k0 + q1.a*k1;
	float qb = q0.b*k0 + q1.b*k1;
	float qc = q0.c*k0 + q1.c*k1;
	float qd = q0.d*k0 + q1.d*k1;

	return new PVector(qb,qc,qd);	
}

public PVector rotateAroundArbitraryAxis(PVector offsetParam,PVector controlParam,PVector axisParam,float angle) {
	PVector control = controlParam.get();
	PVector axis = axisParam.get();
	PVector offset = offsetParam.get();

	control.sub(offset);
	axis.sub(offset);
	axis.normalize(axis);

	float rads = PI/180 * angle;                       
	float sinTheta = sin(rads*.5);
	float cosTheta = cos(rads*.5);

	Quaternion rotQuat   = new Quaternion(  cosTheta,  
	sinTheta * axis.x,  
	sinTheta * axis.y,  
	sinTheta * axis.z );
	Quaternion pointQuat = new Quaternion(  0,
	control.x,
	control.y,
	control.z );
	Quaternion conjQuat  = new Quaternion(  rotQuat.a,
	-rotQuat.b,
	-rotQuat.c,
	-rotQuat.d );

	Quaternion temp = quat_mult(rotQuat,pointQuat);
	PVector result = quat_pointmult(temp,conjQuat);
	result.add(offset);
	return result;
}

public PVector rotateAroundZAxis(PVector offset, PVector controlParam, float angle) {

	PVector control = controlParam.get();
	control.sub(offset);

	float rads = PI/180 * angle;
	float cosTheta = cos(rads);
	float sinTheta = sin(rads);

	float x = cosTheta*control.x + sinTheta*control.y;
	float y = -sinTheta*control.x + cosTheta*control.y;
	float z = control.z;

	PVector result = new PVector(x,y,z);
	result.add(offset);

	return result;
}

public PVector rotateAroundYAxis(PVector offset, PVector controlParam, float angle) {
	PVector control = controlParam.get();
	control.sub(offset);

	float rads = PI/180 * angle;
	float cosTheta = cos(rads);
	float sinTheta = sin(rads);

	float x = cosTheta*control.x - sinTheta*control.z;
	float y = control.y;
	float z = sinTheta*control.x + cosTheta*control.z;

	PVector result = new PVector(x,y,z);

	result.add(offset);

	return result;
}

public PVector rotateAroundXAxis(PVector offset, PVector controlParam, float angle) {
	PVector control = controlParam.get();
	control.sub(offset);

	float rads = PI/180 * angle;
	float cosTheta = cos(rads);
	float sinTheta = sin(rads);

	float x = control.x;
	float y = cosTheta*control.y + sinTheta*control.z;
	float z = -sinTheta*control.y + cosTheta*control.z;

	PVector result = new PVector(x,y,z);

	result.add(offset);

	return result;
}


