class Segment {

	public int seed;
	public float normalizedSeed;

	public float angle;

	public int totalLines = 25;

	public PVector[] baseOrbit = new PVector[totalLines];
	public PVector[] startOrbit = new PVector[totalLines];
	public PVector[] startMarginOrbit = new PVector[totalLines];
	public PVector[] endOrbit = new PVector[totalLines];
	public PVector[] endMarginOrbit = new PVector[totalLines];
	public PVector[] midOrbit = new PVector[totalLines];
	public PVector[] prevStartOrbit = new PVector[totalLines];
	public PVector[] nextStartOrbit = new PVector[totalLines];
	public PVector[] startLerpOrbit = new PVector[totalLines];
	public PVector[] endLerpOrbit = new PVector[totalLines];
	
	public PVector zeroVect;

	public PVector startRelativeVec;
	public PVector endRelativeVec;
	public PVector startCrossRelativeVec;
	public PVector endCrossRelativeVec;
	public PVector pseudoRelativeVec;

	public PVector startAbsoluteVec;
	public PVector endAbsoluteVec;
	public PVector startCrossAbsoluteVec;
	public PVector endCrossAbsoluteVec;
	public PVector pseudoAbsoluteVec;

	public Segment previousSegment;

	public PVector randomDirectionVector;
	public PVector currentDirectionVector;

	PVector velocityVec,xVec,targetVec;
	color from = color(255, 0, 102);
	color to = color(0, 102, 153);

	public float scalar = 1;

	public Segment() {
		zeroVect	= new PVector( 0,0,0);
		endRelativeVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
		currentDirectionVector	= new PVector( 0,0,0);

		velocityVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
		xVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
		targetVec = new PVector(random(-1,1),random(-1,1),random(-1,1));
	}

	public void randomizeDirection(){
		// println("randomizeDirection + " + seed);
		randomDirectionVector = new PVector(random(-1,1),random(-1,1),random(-1,1));
	}
	
	public void createRelatives(){
		// print("createRelatives");
		startRelativeVec = new PVector( 0, 0, 0 );
		
		/*
		PVector dif = PVector.sub(randomDirectionVector,currentDirectionVector);
		dif.mult(.025*(totalSegments-seed)*.1);
		currentDirectionVector.add(dif);

		endRelativeVec = currentDirectionVector.get();
		endRelativeVec.mult(rootRadius);
		// endRelativeVec.mult(rootRadius*(seed));
		*/
		// velocityVec,xVec,targetVec;
		
		PVector divVec = PVector.sub(randomDirectionVector,xVec);
		divVec.mult(elastictySmoothness*normalizedSeed*elastictySpread);
		
		velocityVec = PVector.add(velocityVec,divVec);
		velocityVec = PVector.mult(velocityVec,elastictyRigidity);
		
		xVec = PVector.add(velocityVec,xVec);
		
		endRelativeVec = xVec.get();
		endRelativeVec.mult(rootRadius);
		
		
	}

	public void setControlParams(){
		// scalar = mouseY-height/2;// random(50) + 50;
		float noiseVal = noise(frameCount*.0125,seed*10);
		// scalar = noiseVal*50 + 50;
		scalar = rootScalar + noiseVal*100;
		// scalar = rootScalar*noiseVal;
	}

	public void resetRelatives(){
		pseudoRelativeVec = new PVector( 0, 0, scalar );
		startCrossAbsoluteVec = new PVector( 0, 0, 0 );
		endCrossAbsoluteVec = new PVector( 0, 0, 0 );

		startCrossRelativeVec = endRelativeVec.cross(pseudoRelativeVec);
		startCrossRelativeVec.normalize();
		startCrossRelativeVec.mult(scalar);

		endCrossRelativeVec = startCrossRelativeVec.get();
		endCrossRelativeVec.add(endRelativeVec);

	}
	
	public void createAbsolutes(){
		startAbsoluteVec = startRelativeVec.get();
		endAbsoluteVec = endRelativeVec.get();
		startCrossAbsoluteVec = startCrossRelativeVec.get();
		endCrossAbsoluteVec = endCrossRelativeVec.get();
		pseudoAbsoluteVec = pseudoRelativeVec.get();
	}
	
	public void updateAbsolutes(PVector offset){
		startAbsoluteVec.add(offset);
		endAbsoluteVec.add(offset);
		startCrossAbsoluteVec.add(offset);
		endCrossAbsoluteVec.add(offset);
		pseudoAbsoluteVec.add(offset);
		
	}
	
	public void setPreviousOrbit(PVector[] orbiter){
		prevStartOrbit = orbiter;
	}
	public void setNextOrbit(PVector[] orbiter){
		nextStartOrbit = orbiter;
	}
	
	public void parse() {

		PVector startMarginVec = endRelativeVec.get();
		startMarginVec.mult(margins);
		PVector midVec = endRelativeVec.get();
		midVec.mult(.5);
		
		PVector endMarginVec = endRelativeVec.get();
		endMarginVec.mult(1-margins);
		 
		for(int i=0;i<totalLines;i++) {
			// angle =  360*float(i)/float(totalLines);
			// angle = 360*float(i)/float(totalLines) +360*(1-float(seed)/float(totalLines));
			// angle = 360*float(i)/float(totalLines) +180*(1-float(seed)/float(totalLines))+ frameCount*.125;
			angle = 360*float(i)/float(totalLines) +360*(1-float(seed)/float(totalLines))+ frameCount*rootSpinSpeed*seed;
			
			startOrbit[i] = rotateAroundArbitraryAxis( startAbsoluteVec, startCrossAbsoluteVec, endAbsoluteVec, angle );
			
			startMarginOrbit[i] = startOrbit[i].get();
			startMarginOrbit[i].add(startMarginVec);

			midOrbit[i] = startOrbit[i].get();
			midOrbit[i].add(midVec);
			
			endMarginOrbit[i] = startOrbit[i].get();
			endMarginOrbit[i].add(endMarginVec);
	
			endOrbit[i] = startOrbit[i].get();
			endOrbit[i].add(endRelativeVec);
			
		}         
	}
	
	public void parseLerps() {
		for(int i=0;i<totalLines;i++) {
			startLerpOrbit[i] = lerp3D(prevStartOrbit[i],startOrbit[i],.5);
			endLerpOrbit[i] = lerp3D(nextStartOrbit[i],endOrbit[i],.5);	
			
			// startLerpOrbit[i] = slerp(prevStartOrbit[i],startOrbit[i],.5);
			// endLerpOrbit[i] = slerp(nextStartOrbit[i],endOrbit[i],.5);	
			
		}
	}
	public void drawWireframes() {
		drawCircleOutlineArray(startOrbit,#FFFFFF,102);
		drawCircleOutlineArray(endOrbit,#FFFFFF,102);

		drawCircleOutlineArray(midOrbit,#00FF00,51);
		drawCircleOutlineArray(startMarginOrbit,#FF00FF,51);
		drawCircleOutlineArray(endMarginOrbit,#FF00FF,51);
		
		drawCirclePointArray(startLerpOrbit,#FF0000,102);
		drawCirclePointArray(endLerpOrbit,#FF0000,102);
		
		
		drawLine(startAbsoluteVec,pseudoAbsoluteVec,1,#00FF00,102);

		drawPoint(startCrossAbsoluteVec,3,#FF00FF,255);
		drawLine(startAbsoluteVec,startCrossAbsoluteVec,1,#FF00FF,102);

		drawPoint(endCrossAbsoluteVec,3,#FFFFFF,255);
		drawLine(endAbsoluteVec,endCrossAbsoluteVec,1,#FFFFFF,102);
		
	}
	public void drawShapes() {
		
		int lines = endOrbit.length;
		int i;
		float percentage;
		
		for(i=0;i<lines;i++) {
		
			noFill();
			percentage = 1;//(float(i)/float(lines));
			stroke(#FFFFFF,percentage*25);
			strokeWeight(1);
			// curve(x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4);
						
			bezier(
				startAbsoluteVec.x, startAbsoluteVec.y, startAbsoluteVec.z,
				startLerpOrbit[i].x,startLerpOrbit[i].y,startLerpOrbit[i].z,
				startMarginOrbit[i].x,startMarginOrbit[i].y,startMarginOrbit[i].z,
				midOrbit[i].x,midOrbit[i].y,midOrbit[i].z
			);
			
			bezier(
				endAbsoluteVec.x, endAbsoluteVec.y, endAbsoluteVec.z,
				endLerpOrbit[i].x,endLerpOrbit[i].y,endLerpOrbit[i].z,
				endMarginOrbit[i].x,endMarginOrbit[i].y,endMarginOrbit[i].z,
				midOrbit[i].x,midOrbit[i].y,midOrbit[i].z
			);
		
		}                                                        
	}
	public void drawHighlights() {
		
		int lines = endOrbit.length;
		int i;
		float percentage;
		float bx,by,bz;
		float ax,ay,az;
		PVector pointA,pointB;
		PVector midA,midB;
		color interColor;
		for(i=0;i<lines;i++) {
			
			percentage = (float(frameCount)*rootHighlightSpeed/float(lines))%1;
			bx = bezierPoint(startAbsoluteVec.x, startLerpOrbit[i].x, startMarginOrbit[i].x, midOrbit[i].x, percentage	);
			by = bezierPoint(startAbsoluteVec.y, startLerpOrbit[i].y, startMarginOrbit[i].y, midOrbit[i].y, percentage	);
			bz = bezierPoint(startAbsoluteVec.z, startLerpOrbit[i].z, startMarginOrbit[i].z, midOrbit[i].z, percentage	);
			pointB = new PVector(bx,by,bz);

			interColor = lerpColor(#99FFFF,#FF66FF,  percentage*.5);
			drawPoint(pointB,2,interColor,255);
			midB = lerp3D(startAbsoluteVec,endAbsoluteVec,percentage*.5);
			drawLine(midB,pointB,1,interColor,102);
			
			
			ax = bezierPoint(endAbsoluteVec.x, endLerpOrbit[i].x, endMarginOrbit[i].x, midOrbit[i].x, 1-percentage	);
			ay = bezierPoint(endAbsoluteVec.y, endLerpOrbit[i].y, endMarginOrbit[i].y, midOrbit[i].y, 1-percentage	);
			az = bezierPoint(endAbsoluteVec.z, endLerpOrbit[i].z, endMarginOrbit[i].z, midOrbit[i].z, 1-percentage	);
			pointA = new PVector(ax,ay,az);
			interColor = lerpColor(#99FFFF,#FF66FF,  percentage*.5+.5);
			drawPoint(pointA,2,interColor,255);
			midA = lerp3D(startAbsoluteVec,endAbsoluteVec,percentage*.5+.5);
			drawLine(midA,pointA,1,interColor,102);
			
		}       
	}
	public void drawMesh() {

		PVector[] sideA;
		PVector[] sideB;
		PVector[] sideC;
		PVector[] sideD;

		int lines = endOrbit.length;
		int dotsPerLines = 10;
		int i,j,a,b;
		float percentage;
		float bx,by,bz;
		
		for(i=0;i<lines;i++) {
			
			sideA = new PVector[dotsPerLines];
			sideB = new PVector[dotsPerLines];
			sideC = new PVector[dotsPerLines];
			sideD = new PVector[dotsPerLines];

			a = i%lines;
			b = (i+1)%lines;
	
			// generates points
			for(j=0;j<(dotsPerLines);j++) {

				percentage = (float(j+1)/float(dotsPerLines));

				bx = bezierPoint(startAbsoluteVec.x, startLerpOrbit[a].x, startMarginOrbit[a].x, midOrbit[a].x, percentage	);
				by = bezierPoint(startAbsoluteVec.y, startLerpOrbit[a].y, startMarginOrbit[a].y, midOrbit[a].y, percentage	);
				bz = bezierPoint(startAbsoluteVec.z, startLerpOrbit[a].z, startMarginOrbit[a].z, midOrbit[a].z, percentage	);
				sideA[j] = new PVector(bx,by,bz);

				bx = bezierPoint(startAbsoluteVec.x, startLerpOrbit[b].x, startMarginOrbit[b].x, midOrbit[b].x, percentage	);
				by = bezierPoint(startAbsoluteVec.y, startLerpOrbit[b].y, startMarginOrbit[b].y, midOrbit[b].y, percentage	);
				bz = bezierPoint(startAbsoluteVec.z, startLerpOrbit[b].z, startMarginOrbit[b].z, midOrbit[b].z, percentage	);
				sideB[j] = new PVector(bx,by,bz);
				
				bx = bezierPoint(endAbsoluteVec.x, endLerpOrbit[a].x, endMarginOrbit[a].x, midOrbit[a].x, percentage	);
				by = bezierPoint(endAbsoluteVec.y, endLerpOrbit[a].y, endMarginOrbit[a].y, midOrbit[a].y, percentage	);
				bz = bezierPoint(endAbsoluteVec.z, endLerpOrbit[a].z, endMarginOrbit[a].z, midOrbit[a].z, percentage	);
				sideC[j] = new PVector(bx,by,bz);

				bx = bezierPoint(endAbsoluteVec.x, endLerpOrbit[b].x, endMarginOrbit[b].x, midOrbit[b].x, percentage	);
				by = bezierPoint(endAbsoluteVec.y, endLerpOrbit[b].y, endMarginOrbit[b].y, midOrbit[b].y, percentage	);
				bz = bezierPoint(endAbsoluteVec.z, endLerpOrbit[b].z, endMarginOrbit[b].z, midOrbit[b].z, percentage	);
				sideD[j] = new PVector(bx,by,bz);

			}
			
			noStroke();
			fill(#99FFFF,255);

			beginShape(QUAD_STRIP); 
			vertex(startAbsoluteVec.x,startAbsoluteVec.y,startAbsoluteVec.z);
			vertex(startAbsoluteVec.x,startAbsoluteVec.y,startAbsoluteVec.z);
			for(j=0;j<dotsPerLines;j++) {
				percentage = (float(j+1)/float(dotsPerLines));
				color interColor = lerpColor(#99FFFF,#FF66FF,  percentage);
				fill(interColor,255);
				vertex(sideA[j].x,sideA[j].y,sideA[j].z);
				vertex(sideB[j].x,sideB[j].y,sideB[j].z);
			}
			endShape(CLOSE);

			fill(#99FFFF,255);
			beginShape(QUAD_STRIP);  
			vertex(endAbsoluteVec.x,endAbsoluteVec.y,endAbsoluteVec.z);
			vertex(endAbsoluteVec.x,endAbsoluteVec.y,endAbsoluteVec.z);
			for(j=0;j<dotsPerLines;j++) {
				percentage = (float(j+1)/float(dotsPerLines));
				color interColor = lerpColor(#99FFFF,#FF66FF,  percentage);
				fill(interColor,255);

				vertex(sideC[j].x,sideC[j].y,sideC[j].z);
				vertex(sideD[j].x,sideD[j].y,sideD[j].z);
			}
			endShape(CLOSE);
		}       
	}
	
	public void drawGuides() {
		
		drawPoint(startAbsoluteVec,3,#FFFFFF,255);
		drawPoint(endAbsoluteVec,3,#FFFFFF,255);
		drawLine(startAbsoluteVec,endAbsoluteVec,1,#FFFFFF,153);
	}
	
	void drawCirclePointArray(PVector[] orbiter, color colorParam,float opacityParam) {
		PVector curr = new PVector(0,0,0);
		int lines = orbiter.length;
		int i;
		float percentage;

		// drawLine(prev,curr,1,#FFFFFF,51);

		for(i=0;i<lines;i++) {
			curr = orbiter[i].get();
			percentage = 1;//(float(i)/float(lines));
			drawPoint(curr,1,colorParam,percentage*opacityParam);
		}
	}
	void drawCircleOutlineArray(PVector[] orbiter, color colorParam,float opacityParam) {
		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = orbiter.length;
		int i;
		float percentage;

		prev = orbiter[0].get();
		curr = orbiter[lines-1].get();

		for(i=1;i<lines;i++) {
			prev = curr.get();
			curr = orbiter[i].get();
			percentage = 1;//(float(i)/float(lines));
			drawLine(prev,curr,1,colorParam,percentage*opacityParam);
		}
		
	}
	
	void drawWidthArray(PVector[] orbiter, color colorParam) {

		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = orbiter.length;
		float percentage;
		float colorShifter = sin( ((normalizedSeed + frameCount*.001) * TWO_PI))*.5+.5;
		strokeWeight(1);
		color interColor = lerpColor(from, to,  colorShifter);

		for(int i=1;i<lines;i++) {
			curr = orbiter[i].get();
			prev = orbiter[i-1].get();

			percentage = (float(i)/float(lines)) * 255;

			noStroke();

			fill(interColor, percentage);
			beginShape(TRIANGLE_FAN);
			vertex(prev.x,prev.y,prev.z);
			vertex(curr.x,curr.y,curr.z);
			vertex(startAbsoluteVec.x,startAbsoluteVec.y,startAbsoluteVec.z);
			vertex(prev.x,prev.y,prev.z);
			endShape();

		}
	}

	public void drawPoint(PVector pointParam, int strokeParam, color colorParam, float opacityParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,opacityParam);
		point( pointParam.x,pointParam.y,pointParam.z);
	}

	public void drawLine(PVector pointAParam,PVector pointBParam, float strokeParam, color colorParam, float opacityParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,opacityParam);
		line( pointAParam.x,pointAParam.y,pointAParam.z,pointBParam.x,pointBParam.y,pointBParam.z);
	}
}

